local toc, privateData = ...
local AddonId = toc.identifier

EM = {}
EM.Event = {}
EM.Event.Trigger = {}
EM.Mail = {}
EM.Mail.List = {}
EM.UI = {}
EM.UI.Created = false
EM.UI.Callback = {}
EM.Messages = {}
EM.Temp = {}
EM.Context = {}

EM.Context.Main = UI.CreateContext("emContextMain")

EM.PlayerName = Inspect.Unit.Detail('player').name .. "@" .. Inspect.Shard().name

EM.Event.Trigger.RefreshMail, EM.Event.RefreshMail = Utility.Event.Create(AddonId, "RefreshMail")
EM.Event.Trigger.UICreated, EM.Event.UICreated = Utility.Event.Create(AddonId, "UICreated")

function EM.UI.Create()
	
	local window = UI.CreateFrame("SimpleWindow", "emMailWindow", EM.Context.Main)
		window:SetCloseButtonVisible(true)
		window:SetPoint("TOPLEFT", UI.Native.Mail, "TOPLEFT", 0, -8)
		window:SetWidth(456)
		window:SetHeight(597)
		window:SetLayer(50)
		window:SetTitle("MAIL")
		window:SetVisible(true)
		window.Event.Close = function ()
			EM.UI.Tabs:setActivePane(1)
			if EM.UI.ContactsWindow then
				EM.UI.ContactsWindow:SetVisible(false)
			end
			if EM.UI.Send.element.Display then
				EM.Utility.ClearFocus()
			end
		end
		
	EM.UI.MailWindow = window
	EM.Context.Main:SetStrata("tutorial")
	
	EM.UI.Inbox = NK.CreateFrame("nkExtFrame", "emMailInboxTab", EM.UI.MailWindow)
	EM.UI.Send = NK.CreateFrame("nkExtFrame", "emMailSendTab", EM.UI.MailWindow)
	EM.UI.Archive = NK.CreateFrame("nkExtFrame", "emMailArchiveTab", EM.UI.MailWindow)
	EM.UI.Settings = NK.CreateFrame("nkExtFrame", "emMailSettingsTab", EM.UI.MailWindow)
	
	local tabs = NK.CreateFrame("nkExtTabPane", "emMailWindowTabs", EM.UI.MailWindow,
		{	height = 526, width = 428, orientation = 'horizontal',
			color = { label = "D8CC9E" },
			anchors = {{ from = "TOPLEFT", object = EM.UI.MailWindow, to = "TOPLEFT", x = 14, y = 58 }},
			headerWidth = 104, headerHeight = 29, headerOffsetX = 2.5,
			bodyTexture = 'gfx/tabGridBGShort.png', paneTextureAddon = 'EasyMail',
			paneTextureInactive = 'nkGenieEM/gfx/tabPaneHorizInActive.png',
			paneTextureActive = 'nkGenieEM/gfx/tabPaneHorizActive.png',
			
			panes = {{content = EM.UI.Inbox, label = "INBOX", initFunc = function () EM.UI.SetupInboxTab() end, switchFunc = function () for child, st in pairs(EM.Context.Inventory:GetChildren()) do child:SetVisible(false) end end},
					 {content = EM.UI.Send, label = "SEND", initFunc = function () EM.UI.SetupSendTab() end, switchFunc = function () for child, st in pairs(EM.Context.Inventory:GetChildren()) do child:SetVisible(true) end end},
					 {content = EM.UI.Archive, label = "ARCHIVE", initFunc = function () EM.UI.SetupArchiveTab() end, switchFunc = function () for child, st in pairs(EM.Context.Inventory:GetChildren()) do child:SetVisible(false) end end},
					 {content = EM.UI.Settings, label = "SETTINGS", initFunc = function () EM.UI.SetupSettingsTab() end, switchFunc = function () for child, st in pairs(EM.Context.Inventory:GetChildren()) do child:SetVisible(false) end end}}
		})
	EM.UI.Tabs = tabs
end

local function PopulateMail(messageId, t)
	local basic = Inspect.Mail.Detail(messageId)
	if basic then
		local message = EM.Mail.Create(messageId)
		for k,v in pairs(basic) do
			if k == "attachments" and message["attachments"] then 
				message["attachments"] = nil 
			end
			message[k] = v
		end
		message["expire"] = os.date("%x %X", message["expire"])
		if not message["attachmentCount"] then
			if t == "basic" and message["attachments"] then
				message["attachmentCount"] = message["attachments"]
			elseif message["attachments"] then
				message["attachmentCount"] = #message["attachments"]
			end
		end
		if t == "detail" and message["cod"] then
			message["paidCoD"] = message["cod"]
		elseif t == "detail" and EM.Messages[messageId].paidCoD then
			message["paidCoD"] = EM.Messages[messageId].paidCoD
		end
		return message
	else
		return nil
	end
end

function EM.Mail.Create(messageId)
	local message = {}
	setmetatable(message, self)
	message.id = messageId
	return message
end

local function MailAvailable(mail)

	if not EM.UI.Created then return end

	EM.Mail.List.Original = mail
	for id,t in pairs(EM.Mail.List.Original) do
		if t and not EM.Messages[id] then
			EM.Messages[id] = PopulateMail(id, t)
			EasyMailTable[EM.PlayerName].Inbox[id] = EM.Messages[id]
		elseif t == "detail" and EM.Messages[id] then
			EM.Messages[id] = PopulateMail(id, t)
			if EM.Messages[id]["attachmentCount"] ~= 0 then
				EasyMailTable[EM.PlayerName].Inbox[id] = EM.Messages[id]
			end
		elseif not t then
			EasyMailTable[EM.PlayerName].Inbox[id].deleted = os.date("%x %X", Inspect.Time.Server())
			EM.Messages[id] = nil
			EM.Event.Trigger.RefreshMail(id, t)
			return
		end
	end
	EM.Event.Trigger.RefreshMail()
end

local function AddonLoaded()
	Command.Console.Display("general", false, "<font color='#4BCDD6'>EasyMail Loaded</font>", true)
end

local function SavedDataLoaded(id)
	if id == AddonId then
		if type(_G["EasyMail"]) == "string" then
			EasyMailTable = loadstring("return " .. zlib.inflate()(_G["EasyMail"]))
			EasyMailTable = EasyMailTable and EasyMailTable() or {}
		else
			EasyMailTable = {}
		end
		EasyMailTable[EM.PlayerName] = EasyMailTable[EM.PlayerName] or {PlayerID = Inspect.Unit.Lookup('player')}
		EasyMailTable[EM.PlayerName].Inbox = EasyMailTable[EM.PlayerName].Inbox or {}
		EasyMailTable[EM.PlayerName].Sent = EasyMailTable[EM.PlayerName].Sent or {}
	end
end

local function SaveDataBegin(id)
	if id == AddonId then
		_G["EasyMail"] = zlib.deflate(zlib.BEST_COMPRESSION)(Utility.Serialize.Inline(EasyMailTable), "finish")
	end
end

function EM.InteractionChanged(interaction, state)
	if interaction == "mail" and state and not EM.UI.MailWindow then
		Command.System.Watchdog.Quiet()
		EM.UI.Create()
		EM.UI.Inbox:getElement().Grid:getElement():SetVisible(false)
	elseif interaction == "mail" and state and EM.UI.MailWindow then
		EM.UI.MailWindow:SetVisible(true)
		EM.Context.Main:SetStrata("tutorial")
	elseif interaction == "mail" and not state then
		EM.UI.Tabs:setActivePane(1)
		if EM.UI.ContactsWindow then
			EM.UI.ContactsWindow:SetVisible(false)
		end
		EM.UI.MailWindow:SetVisible(false)
		if EM.UI.PopupWindow then
			EM.UI.PopupWindow:SetVisible(false)
		end
		if EM.UI.Inbox.Grid.markedRows then
			for row, data in ipairs(EM.UI.Inbox.Grid.markedRows) do
				local id = EM.UI.Inbox.Grid.values[data.row][1].key
				if data.status == 'true' then
					EM.UI.Inbox.Grid:rowMark(data.location, false)
					EM.UI.Inbox.Grid.markedCount = EM.UI.Inbox.Grid.markedCount - 1
				end
			end
		end
		for child, st in pairs(EM.Context.Inventory:GetChildren()) do
			child:SetVisible(false)
		end
		EM.UI.Inbox.element.btnTakeSelected:SetEnabled(false)
		EM.UI.Inbox.element.btnDeleteSelected:SetEnabled(false)
	end
end

table.insert(Event.Interaction,						{ EM.InteractionChanged, AddonId, AddonId .. "_InteractionChanged" })
table.insert(Event.Mail, 							{ MailAvailable, AddonId, AddonId .. "_MailAvailable" })
table.insert(Event.Addon.Startup.End,				{ AddonLoaded, AddonId, AddonId .. "_AddonLoaded" })
table.insert(Event.Addon.SavedVariables.Load.End,	{ SavedDataLoaded, AddonId, AddonId .. "_SavedDataLoaded" })
table.insert(Event.Addon.SavedVariables.Save.Begin, { SaveDataBegin, AddonId, AddonId .. "_SavedDataLoaded" })
table.insert(EM.Event.UICreated,					{ MailAvailable, AddonId, AddonId .. "_UICreated" })